@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013,2017 MariaDB Corporation AB
+   Copyright (C) 2013,2018 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,8 +20,9 @@
 #define _ma_helper_h_
 
 void CloseMultiStatements(MADB_Stmt *Stmt);
-BOOL QueryIsPossiblyMultistmt(char *queryStr);
-unsigned int GetMultiStatements(MADB_Stmt *Stmt, char *StmtStr, SQLINTEGER Length);
+BOOL QueryIsPossiblyMultistmt(MADB_QUERY *Query);
+int  SqlRtrim(char *StmtStr, int Length);
+unsigned int GetMultiStatements(MADB_Stmt *Stmt);
 int MADB_KeyTypeCount(MADB_Dbc *Connection, char *TableName, int KeyFlag);
 MYSQL_RES *MADB_ReadDefaultValues(MADB_Dbc *Dbc, const char *Catalog, const char *TableName);
 int MADB_GetDefaultType(int SQLDataType);
@@ -45,8 +46,11 @@ size_t MADB_GetHexString(char *BinaryBuffer, size_t BinaryLength,
 
 size_t MADB_GetDisplaySize(MYSQL_FIELD field, CHARSET_INFO *charset);
 size_t MADB_GetOctetLength(MYSQL_FIELD Field, unsigned short MaxCharLen);
-char *MADB_GetTypeName(MYSQL_FIELD Field);
-char *trim(char *Str);
+char * MADB_GetTypeName(MYSQL_FIELD Field);
+
+char * ltrim(char *Str);
+char * trim(char *Str);
+
 my_bool MADB_CheckPtrLength(SQLINTEGER MaxLength, char *Ptr, SQLINTEGER NameLen);
 void *GetBindOffset(MADB_Desc *Ard, MADB_DescRecord *ArdRecord, void *Ptr, SQLULEN RowNumber, size_t PtrSize);
 BOOL MADB_ColumnIgnoredInAllRows(MADB_Desc *Desc, MADB_DescRecord *Rec);
@@ -56,12 +60,13 @@ MYSQL_RES *MADB_GetDefaultColumnValues(MADB_Stmt *Stmt, MYSQL_FIELD *fields);
 char *MADB_GetDefaultColumnValue(MYSQL_RES *res, const char *Column);
 
 /* SQL_NUMERIC stuff */
-int MADB_CharToSQLNumeric(char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord, unsigned long RowNumber);
-size_t MADB_SqlNumericToChar(SQL_NUMERIC_STRUCT *Numeric, char *Buffer, int *ErrorCode);
-void MADB_NumericInit(SQL_NUMERIC_STRUCT *number, MADB_DescRecord *Ard);
-unsigned long MADB_StmtDataTell(MADB_Stmt *Stmt);
-BOOL MADB_IsNumericType(SQLSMALLINT ConciseType);
-BOOL MADB_IsIntType(SQLSMALLINT ConciseType);
+int           MADB_CharToSQLNumeric (char *buffer, MADB_Desc *Ard, MADB_DescRecord *ArdRecord,
+                                     SQL_NUMERIC_STRUCT *dst_buffer, unsigned long RowNumber);
+size_t        MADB_SqlNumericToChar (SQL_NUMERIC_STRUCT *Numeric, char *Buffer, int *ErrorCode);
+void          MADB_NumericInit      (SQL_NUMERIC_STRUCT *number, MADB_DescRecord *Ard);
+
+unsigned long MADB_StmtDataTell         (MADB_Stmt *Stmt);
+int           MADB_FindNextDaeParam     (MADB_Desc *Desc, int InitialParam, SQLSMALLINT RowNumber);
 
 BOOL          MADB_IsNumericType(SQLSMALLINT ConciseType);
 BOOL          MADB_IsIntType    (SQLSMALLINT ConciseType);
