@@ -282,11 +282,6 @@ enum MADB_StmtState { MADB_SS_INITED= 0, MADB_SS_EMULATED= 1, MADB_SS_PREPARED=
 #define STMT_WAS_PREPARED(Stmt_Hndl) (Stmt_Hndl->State > MADB_SS_EMULATED)
 #define RESET_STMT_STATE(Stmt_Hndl) Stmt_Hndl->State= STMT_WAS_PREPARED(Stmt_Hndl) ? MADB_SS_PREPARED : MADB_SS_INITED
 
-
-typedef struct {
-  DYNAMIC_ARRAY tokens;
-} MADB_QUERY;
-
 /* Struct used to define column type when driver has to fix it (in catalog functions + SQLGetTypeInfo) */
 typedef struct
 {
@@ -297,6 +292,11 @@ typedef struct
 
 } MADB_ShortTypeInfo;
 
+/* Stmt struct needs definitions from my_parse.h */
+#include <ma_parse.h>
+
+#define STMT_STRING(STMT) (STMT)->Query.Original
+
 struct st_ma_odbc_stmt
 {
   MADB_Dbc                  *Connection;
@@ -307,7 +307,7 @@ struct st_ma_odbc_stmt
   MYSQL_STMT                *stmt;
   MYSQL_RES                 *metadata;
   LIST                      ListItem;
-  MADB_QUERY                *Tokens;
+  MADB_QUERY                Query;
   SQLSMALLINT               ParamCount;
   enum MADB_DaeType         DataExecutionType;
   MYSQL_RES                 *DefaultsResult;
@@ -316,11 +316,9 @@ struct st_ma_odbc_stmt
   int                       Status;
   MADB_DescRecord           *PutDataRec;
   MADB_Stmt                 *DaeStmt;
-  char                      *StmtString;
   MADB_Stmt                 *PositionedCursor;
-  unsigned int              PositionedCommand;
+  my_bool                   PositionedCommand;
   enum MADB_StmtState       State;
-  unsigned int              MultiStmtCount;
   MYSQL_STMT                **MultiStmts;
   unsigned int              MultiStmtNr;
   unsigned int              MultiStmtMaxParam;
@@ -380,7 +378,11 @@ struct st_ma_odbc_connection
   MADB_Dsn *Dsn;
   struct st_ma_connection_methods *Methods;
   MADB_Error Error;
-  Client_Charset charset;
+
+  Client_Charset Charset;
+  Client_Charset *ConnOrSrcCharset; /* "Source" here stands for which charset Windows DM was using as source, when converted to unicode.
+                                  We have to use same charset to recode from unicode to get same string as application sent it.
+                                  For Unicode application that is the same as "Charset", or in case of ANSI on Windows - defaulst system codepage */
   char *DataBase;
   LIST ListItem;
   LIST *Stmts;
@@ -420,9 +422,9 @@ typedef struct
 SQLRETURN DSNPrompt_Lookup(MADB_Prompt *prompt, const char *SetupLibName, MADB_Dbc *Dbc);
 int       DSNPrompt_Free  (MADB_Prompt *prompt);
 
-Client_Charset* GetDefaultOsCharset(Client_Charset *cc);
 int             InitClientCharset  (Client_Charset *cc, const char * name);
-void            CloseClientCharset (Client_Charset *cc);
+void            CopyClientCharset(Client_Charset * Src, Client_Charset * Dst);
+void            CloseClientCharset(Client_Charset *cc);
 
 /* Default precision of SQL_NUMERIC */
 #define MADB_DEFAULT_PRECISION 38
